"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _path = require("path");

var _rimraf = require("rimraf");

var _build = _interopRequireDefault(require("af-webpack/build"));

var _umiPlugin = require("umi-plugin");

var _registerBabel = _interopRequireDefault(require("./registerBabel"));

var _getWebpackConfig = _interopRequireDefault(require("./getWebpackConfig"));

var _generateHTML = _interopRequireDefault(require("./generateHTML"));

var _generateEntry2 = _interopRequireDefault(require("./generateEntry"));

var _getRouteConfig = _interopRequireDefault(require("./getRouteConfig"));

var _send = _interopRequireWildcard(require("./send"));

var _getConfig2 = require("./getConfig");

var _getUserConfig = _interopRequireDefault(require("af-webpack/getUserConfig"));

var _getPaths = _interopRequireDefault(require("./getPaths"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = require('debug')('umi-build-dev:build');

function _default(opts = {}) {
  const _opts$cwd = opts.cwd,
        cwd = _opts$cwd === void 0 ? process.cwd() : _opts$cwd,
        babel = opts.babel,
        extraResolveModules = opts.extraResolveModules,
        hash = opts.hash,
        _opts$libraryName = opts.libraryName,
        libraryName = _opts$libraryName === void 0 ? 'umi' : _opts$libraryName,
        _opts$staticDirectory = opts.staticDirectory,
        staticDirectory = _opts$staticDirectory === void 0 ? 'static' : _opts$staticDirectory,
        _opts$tmpDirectory = opts.tmpDirectory,
        tmpDirectory = _opts$tmpDirectory === void 0 ? `.${libraryName}` : _opts$tmpDirectory,
        _opts$outputPath = opts.outputPath,
        outputPath = _opts$outputPath === void 0 ? './dist' : _opts$outputPath,
        pluginFiles = opts.plugins,
        preact = opts.preact;
  const plugins = (0, _umiPlugin.resolvePlugins)(pluginFiles);
  const paths = (0, _getPaths.default)({
    cwd,
    tmpDirectory,
    outputPath
  });

  function getChunkToFilesMap(chunks) {
    return chunks.reduce((memo, chunk) => {
      memo[chunk.name] = chunk.files;
      return memo;
    }, {});
  }

  return new Promise(resolve => {
    // 为配置注册 babel 解析
    (0, _registerBabel.default)(babel, {
      configOnly: true
    }); // 获取 .webpackrc 配置

    const _getWebpackRCConfig = (0, _getUserConfig.default)({
      cwd,
      disabledConfigs: ['entry', 'outputPath', 'hash']
    }),
          webpackRCConfig = _getWebpackRCConfig.config; // 获取用户配置


    const _getConfig = (0, _getConfig2.getConfig)(cwd),
          config = _getConfig.config;

    debug(`清理临时文件夹 ${paths.tmpDirPath}`);
    (0, _rimraf.sync)(paths.absTmpDirPath);
    debug(`清理 outputPath ${paths.outputPath}`);
    (0, _rimraf.sync)(paths.absOutputPath); // 生成入口文件

    debug(`libraryName: ${libraryName}`);

    const _generateEntry = (0, _generateEntry2.default)({
      cwd,
      config,
      plugins,
      routerTpl: opts.routerTpl,
      entryJSTpl: opts.entryJSTpl,
      libraryName,
      paths
    }),
          routeConfig = _generateEntry.routeConfig; // 获取 webpack 配置


    const webpackConfig = (0, _getWebpackConfig.default)({
      cwd,
      config,
      webpackRCConfig,
      babel,
      hash,
      extraResolveModules,
      routeConfig,
      libraryName,
      staticDirectory,
      paths,
      preact
    }); // af-webpack build

    (0, _build.default)({
      webpackConfig,

      success({
        stats
      }) {
        if (!process.env.DISABLE_RM_TMPDIR) {
          debug(`删除临时文件夹 ${paths.tmpDirPath}`);
          (0, _rimraf.sync)(paths.absTmpDirPath);
        }

        debug('打包 HTML...');
        const chunkToFilesMap = getChunkToFilesMap(stats.compilation.chunks);
        debug(`chunkToFilesMap: ${JSON.stringify(chunkToFilesMap)}`);
        const routeConfig = (0, _getRouteConfig.default)(paths.absPagesPath);
        (0, _generateHTML.default)({
          cwd,
          routeConfig,
          config,
          chunkToFilesMap,
          plugins,
          staticDirectory,
          libraryName,
          paths
        });
        debug('打包 HTML 完成...');
        (0, _umiPlugin.applyPlugins)(plugins, 'buildSuccess', null, {
          routeConfig,
          cwd
        });
        (0, _send.default)({
          type: _send.BUILD_DONE
        });
        resolve();
      }

    });
  });
}

module.exports = exports["default"];