"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = generateEntry;
exports.watchPages = watchPages;

var _path = require("path");

var _mkdirp = require("mkdirp");

var _fs = require("fs");

var _chokidar = _interopRequireDefault(require("chokidar"));

var _chalk = _interopRequireDefault(require("chalk"));

var _lodash = _interopRequireDefault(require("lodash.debounce"));

var _umiPlugin = require("umi-plugin");

var _getRouteConfig = _interopRequireDefault(require("./getRouteConfig"));

var _getRouterContent = _interopRequireDefault(require("./getRouterContent"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const debug = require('debug')('umi-build-dev:generateEntry');

let cachedRouterContent = null;
let libraryJSGenerated = false;

function generateEntry(opts = {}) {
  const cwd = opts.cwd,
        onChange = opts.onChange,
        paths = opts.paths;
  const absTmpDirPath = paths.absTmpDirPath;
  (0, _mkdirp.sync)(absTmpDirPath);

  const _generate = generate(opts),
        routeConfig = _generate.routeConfig;

  if (onChange) onChange(routeConfig);
  let hasError = false;

  function rebuild(devServer) {
    try {
      const _generate2 = generate(opts),
            routeConfig = _generate2.routeConfig;

      if (onChange) onChange(routeConfig);

      if (hasError) {
        // 从出错中恢复时，刷新浏览器
        devServer.sockWrite(devServer.sockets, 'content-changed');
        hasError = false;
      }
    } catch (e) {
      // 向浏览器发送出错信息
      devServer.sockWrite(devServer.sockets, 'errors', [e.message]);
      hasError = true;
      cachedRouterContent = null;
      debug(`generate failed: ${e.message}`);
      debug(e);
      console.error(_chalk.default.red(e.message));
    }
  }

  function watch(devServer) {
    watchPages({
      cwd,
      paths,
      onChange: rebuild.bind(null, devServer)
    });
  }

  return {
    watch,
    routeConfig,
    rebuild
  };
}

function watchPages(opts = {}) {
  const cwd = opts.cwd,
        onChange = opts.onChange,
        paths = opts.paths;

  const watcher = _chokidar.default.watch(paths.absPagesPath, {
    ignored: /(^|[\/\\])\../,
    // ignore .dotfiles
    ignoreInitial: true
  });

  watcher.on('all', (0, _lodash.default)((event, path) => {
    debug(`${event} ${path}`);
    onChange(watcher);
  }, 100));
  process.on('SIGINT', () => {
    if (watcher) watcher.close();
  });
  return watcher;
}

function generate(opts = {}) {
  const paths = opts.paths,
        plugins = opts.plugins,
        libraryName = opts.libraryName,
        config = opts.config;
  const absTmpDirPath = paths.absTmpDirPath,
        absPagesPath = paths.absPagesPath;
  const routeConfig = (0, _getRouteConfig.default)(absPagesPath);
  (0, _umiPlugin.applyPlugins)(plugins, 'generateEntry', null, opts); // 缓存一次

  const routerContent = (0, _getRouterContent.default)(_extends({}, opts, {
    routeConfig,
    libraryName,
    config
  }));

  if (cachedRouterContent !== routerContent) {
    (0, _fs.writeFileSync)((0, _path.join)(absTmpDirPath, 'router.js'), routerContent, 'utf-8');
    cachedRouterContent = routerContent;
  } // library js 不会变化，生成一次即可


  if (process.env.DISABLE_UMIJS_G_CACHE || !libraryJSGenerated) {
    (0, _fs.writeFileSync)((0, _path.join)(absTmpDirPath, `${libraryName}.js`), (0, _fs.readFileSync)(opts.entryJSTpl || (0, _path.join)(__dirname, '../template/entry.js')), 'utf-8');
    libraryJSGenerated = true;
  }

  return {
    routeConfig
  };
}