"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getConfig = getConfig;
exports.watchConfigs = watchConfigs;

var _fs = require("fs");

var _path = require("path");

var _reactDevUtils = require("af-webpack/react-dev-utils");

var _chalk = _interopRequireDefault(require("chalk"));

var _lodash = _interopRequireDefault(require("lodash.isequal"));

var _didyoumean = _interopRequireDefault(require("didyoumean"));

var _requireindex = _interopRequireDefault(require("requireindex"));

var _constants = require("../constants");

var _watch2 = require("./watch");

var _createRouteMiddleware = require("../createRouteMiddleware");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const pluginsMap = (0, _requireindex.default)((0, _path.join)(__dirname, './configPlugins'));
const plugins = Object.keys(pluginsMap).map(key => {
  return pluginsMap[key]();
});
let devServer = null;

function printError(messages) {
  if (devServer) {
    devServer.sockWrite(devServer.sockets, 'errors', typeof messages === 'string' ? [messages] : messages);
  }
}

function printWarn(messages) {
  if (devServer) {
    devServer.sockWrite(devServer.sockets, 'warns', typeof messages === 'string' ? [messages] : messages);
  }
}

function reload() {
  if (devServer) {
    devServer.sockWrite(devServer.sockets, 'content-changed');
  }
}

function restart(why) {
  if (devServer) {
    (0, _reactDevUtils.clearConsole)();
    console.log(_chalk.default.green(`Since ${why}, try to restart server`));
    (0, _watch2.unwatch)();
    devServer.close();
    process.send({
      type: 'RESTART'
    });
  }
}

function getConfig(cwd, opts = {}) {
  const files = _constants.CONFIG_FILES.filter(file => (0, _fs.existsSync)((0, _path.join)(cwd, file))).map(file => (0, _path.join)(cwd, file)); // 从两个地方读取配置，但只允许有一个


  if (files.length === 2) {
    printWarn(`
你有两个配置文件，${files[0]} 和 ${files[1]}，请删除其中一个，我们会使用其中的 ${files[0]}。
    `.trim());
  }

  let file = null;
  let config = {}; // 执行插件方法时的上下文

  const context = {
    cwd,
    file,
    printError,
    printWarn,
    restart,
    reload
  };

  if (files.length) {
    file = files[0];
    const relativeFile = file.replace(`${cwd}/`, '');
    context.relativeFile = relativeFile; // 强制读取，不走 require 缓存

    if (opts.force) {
      _constants.CONFIG_FILES.forEach(configFile => {
        delete require.cache[(0, _path.join)(cwd, configFile)];
      });
    }

    try {
      config = require(file); // eslint-disable-line
    } catch (e) {
      const msg = `配置文件 "${relativeFile}" 解析出错，请检查语法。
\r\n${e.toString()}`;
      printError(msg);
      throw new Error(msg);
    } // 把外层 context 的内容复制进来，用内层的覆盖它


    if (config.context && config.pages) {
      Object.keys(config.pages).forEach(key => {
        const page = config.pages[key];
        page.context = _extends({}, config.context, page.context);
      });
    } // 校验


    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = plugins[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        const plugin = _step.value;
        const name = plugin.name;

        if (config[name] && plugin.validate) {
          try {
            plugin.validate.call(_extends({}, context, {
              plugin,
              name,
              watch: _watch2.watch.bind(null, name),
              unwatch: _watch2.unwatch.bind(null, name)
            }), config[name], config);
          } catch (e) {
            // 校验出错后要把值设到缓存的 config 里，确保 watch 判断时才能拿到正确的值
            if (opts.setConfig) {
              opts.setConfig(config);
            }

            printError(e.message);
            throw new Error(`配置 ${name} 校验失败, ${e.message}`);
          }
        }
      } // 找下不匹配的 name

    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return != null) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    const pluginNames = plugins.map(p => p.name);
    Object.keys(config).forEach(key => {
      if (!pluginNames.includes(key)) {
        if (opts.setConfig) {
          opts.setConfig(config);
        }

        const affixmsg = `选择 "${pluginNames.join(', ')}" 中的一项，详见 https://fengdie.alipay-eco.com/doc/h5app/configuration`;
        const guess = (0, _didyoumean.default)(key, pluginNames);
        const midMsg = guess ? `你是不是想配置 "${guess}" ？ 或者` : '请';
        const msg = `"${relativeFile}" 中配置的 "${key}" 并非约定的配置项，${midMsg}${affixmsg}`;
        printError(msg);
        throw new Error(msg);
      }
    });
  }

  let configFailed = false;
  return {
    file,
    config,
    watch: _devServer => {
      devServer = _devServer; // 配置插件的监听

      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = plugins[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          const plugin = _step2.value;
          const name = plugin.name;

          if (plugin.watch) {
            plugin.watch.call(_extends({}, context, {
              plugin,
              name,
              watch: _watch2.watch.bind(null, name),
              unwatch: _watch2.unwatch.bind(null, name)
            }), config[name], config);
          }
        } // 配置文件的监听

      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return != null) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }

      (0, _watch2.watch)('CONFIG_FILES', _constants.CONFIG_FILES).on('all', (event, path) => {
        console.log(`[DEBUG] [${event}] ${path}`);

        try {
          const _getConfig = getConfig(cwd, {
            force: true,

            setConfig(newConfig) {
              config = newConfig;
            }

          }),
                newConfig = _getConfig.config; // 更新 middleware 的配置


          (0, _createRouteMiddleware.setConfig)(newConfig); // 从失败中恢复过来，需要 reload 一次

          if (configFailed) {
            configFailed = false;
            reload();
          }

          var _iteratorNormalCompletion3 = true;
          var _didIteratorError3 = false;
          var _iteratorError3 = undefined;

          try {
            for (var _iterator3 = plugins[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
              const plugin = _step3.value;
              const name = plugin.name,
                    onChange = plugin.onChange;

              if (!(0, _lodash.default)(newConfig[name], config[name])) {
                if (onChange) {
                  onChange.call(_extends({}, context, {
                    plugin,
                    name,
                    watch: _watch2.watch.bind(null, name),
                    unwatch: _watch2.unwatch.bind(null, name)
                  }), newConfig[name], config[name], newConfig);
                }
              }
            }
          } catch (err) {
            _didIteratorError3 = true;
            _iteratorError3 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion3 && _iterator3.return != null) {
                _iterator3.return();
              }
            } finally {
              if (_didIteratorError3) {
                throw _iteratorError3;
              }
            }
          }

          config = newConfig;
        } catch (e) {
          configFailed = true;
          console.error(_chalk.default.red(`watch handler failed, since ${e.message}`));
          console.error(e);
        }
      });
    }
  };
}

function watchConfigs() {
  return (0, _watch2.watch)('CONFIG_FILES', _constants.CONFIG_FILES);
}