"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getRouteConfig;

var _fs = require("fs");

var _path = require("path");

var _constants = require("./constants");

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const DOT_JS = '.js';
const EXT_NAMES = ['.js', '.jsx', '.ts', '.tsx'];

function getRouteConfig(root, dirPath = '') {
  const path = (0, _path.join)(root, dirPath);
  const files = (0, _fs.readdirSync)(path);
  return files.reduce((memo, file) => {
    // 包含 ., .., 以及其他 dotfile
    if (file.charAt(0) === '.') return memo;
    const stats = (0, _fs.statSync)((0, _path.join)(path, file));
    const ext = (0, _path.extname)(file);

    if (stats.isFile() && EXT_NAMES.indexOf(ext) > -1) {
      const fullPath = (0, _path.join)(dirPath, (0, _path.basename)(file, ext));
      return _extends({}, memo, {
        [`/${fullPath}.html`]: `${fullPath}${ext}`
      });
    } else if (stats.isDirectory()) {
      const fullPath = (0, _path.join)(dirPath, file);
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = _constants.ROUTE_FILES[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          const routeFile = _step.value;

          if ((0, _fs.existsSync)((0, _path.join)(root, fullPath, routeFile))) {
            if ((0, _fs.existsSync)((0, _path.join)(root, `${fullPath}${DOT_JS}`))) {
              throw new Error(`路由冲突，src/page 目录下同时存在 "${fullPath}${DOT_JS}" 和 "${(0, _path.join)(fullPath, routeFile)}"，两者指向同一路由。`);
            }

            return _extends({}, memo, {
              [`/${fullPath}.html`]: (0, _path.join)(fullPath, routeFile)
            });
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return != null) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      return _extends({}, memo, getRouteConfig(root, fullPath));
    } else {
      return memo;
    }
  }, {});
} // TODO：dev 模式下如果每次修改 pages 下的内容都重新生成配置，可能会有性能问题


module.exports = exports["default"];