"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getRouterContent;

var _fs = require("fs");

var _path = require("path");

var _normalizeEntry = _interopRequireDefault(require("./normalizeEntry"));

var _winPath = _interopRequireDefault(require("./winPath"));

var _requestCache = require("./requestCache");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = require('debug')('umi-build-dev:getRouterContent');

const isDev = process.env.NODE_ENV === 'development';

function getRouterContent(opts = {}) {
  const routeConfig = opts.routeConfig,
        _opts$tplPath = opts.tplPath,
        tplPath = _opts$tplPath === void 0 ? (0, _path.join)(__dirname, `../template/router.js`) : _opts$tplPath,
        libraryName = opts.libraryName,
        config = opts.config,
        paths = opts.paths;

  if (!(0, _fs.existsSync)(tplPath)) {
    throw new Error('tplPath 不存在');
  }

  const tpl = (0, _fs.readFileSync)(tplPath, 'utf-8');
  const routeComponents = getRouteComponents(routeConfig, config, paths);
  return tpl.replace(/<%= routeComponents %>/g, routeComponents).replace(/<%= libraryName %>/g, libraryName);
}

function getRouteComponents(routeConfig, config = {}, paths) {
  if (routeConfig['/index.html']) {
    routeConfig['/'] = routeConfig['/index.html'];
  }

  const loading = config.loading;
  let loadingOpts = '';

  if (loading) {
    loadingOpts = `, { loading: require('${(0, _path.join)(paths.cwd, loading)}').default }`;
  }

  const routerComponents = Object.keys(routeConfig).map(key => {
    const pageJSFile = (0, _winPath.default)((0, _path.join)('..', routeConfig[key]));
    debug(`${JSON.stringify((0, _requestCache.getRequest)())}, key`);

    if (isDev) {
      const component = (0, _requestCache.getRequest)()[key] ? `require('${pageJSFile}').default` : '() => <div>Compiling...</div>';
      return `    <Route exact path="${key}" component={${component}}></Route>`;
    } else {
      return `    <Route exact path="${key}" component={dynamic(() => import(/* webpackChunkName: '${(0, _normalizeEntry.default)(routeConfig[key])}' */'${pageJSFile}')${loadingOpts})}></Route>`;
    }
  });
  return `
<Router history={window.g_history}>
  <Switch>
${routerComponents.join('\n')}
  </Switch>
</Router>
  `.trim();
}

module.exports = exports["default"];