"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _path = require("path");

var _fs = require("fs");

var _getConfig = _interopRequireDefault(require("af-webpack/getConfig"));

var _reactDevUtils = require("af-webpack/react-dev-utils");

var _postcssPluginPx2rem = _interopRequireDefault(require("postcss-plugin-px2rem"));

var _browsers = _interopRequireDefault(require("./defaultConfigs/browsers"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const debug = require('debug')('umi-build-dev:getWebpackConfig');

const env = process.env.NODE_ENV;

function _default(opts = {}) {
  const cwd = opts.cwd,
        config = opts.config,
        webpackRCConfig = opts.webpackRCConfig,
        babel = opts.babel,
        hash = opts.hash,
        routeConfig = opts.routeConfig,
        libraryName = opts.libraryName,
        staticDirectory = opts.staticDirectory,
        extraResolveModules = opts.extraResolveModules,
        paths = opts.paths,
        preact = opts.preact; // entry

  const entryScript = (0, _path.join)(cwd, `./${paths.tmpDirPath}/${libraryName}.js`);
  const setPublicPathFile = (0, _path.join)(__dirname, '../template/setPublicPath.js');
  const hdFile = (0, _path.join)(__dirname, '../template/hd/index.js');
  const compileOnDemandFile = (0, _path.join)(__dirname, '../template/compileOnDemand.js');
  const isDev = env === 'development';
  const initialEntry = config.hd ? [hdFile] : [];
  const entry = isDev ? {
    [libraryName]: [...initialEntry, ...(process.env.DISABLE_HMR ? [] : [_reactDevUtils.webpackHotDevClientPath]), entryScript, compileOnDemandFile]
  } : {
    [libraryName]: [...initialEntry, setPublicPathFile, entryScript]
  };
  const pageCount = isDev ? null : Object.keys(routeConfig).length;
  debug(`pageCount: ${pageCount}`);
  debug(`config: ${JSON.stringify(config)}`); // default react, support config with preact
  // 优先级：用户配置 > preact argument > default (React)

  const preactAlias = {
    react: require.resolve('preact-compat'),
    'react-dom': require.resolve('preact-compat'),
    'create-react-class': require.resolve('preact-compat/lib/create-react-class')
  };
  const reactAlias = {
    react: require.resolve('react'),
    'react-dom': require.resolve('react-dom')
  };
  let preactOrReactAlias = preact ? preactAlias : reactAlias;

  if (config.preact === true) {
    preactOrReactAlias = preactAlias;
  }

  if (config.preact === false) {
    preactOrReactAlias = reactAlias;
  } // 关于为啥放 webpack 而不放 babel-plugin-module-resolver 里
  // 详见：https://tinyletter.com/sorrycc/letters/babel


  const libAlias = {
    'antd-mobile': (0, _path.dirname)(require.resolve('antd-mobile/package')),
    antd: (0, _path.dirname)(require.resolve('antd/package'))
  }; // 支持用户指定 antd 和 antd-mobile 的版本
  // TODO: 出错处理，用户可能指定了依赖，但未指定 npm install

  const pkgPath = (0, _path.join)(cwd, 'package.json');

  if ((0, _fs.existsSync)(pkgPath)) {
    const _require = require(pkgPath),
          _require$dependencies = _require.dependencies,
          dependencies = _require$dependencies === void 0 ? {} : _require$dependencies;

    if (dependencies.antd) {
      libAlias['antd'] = (0, _path.dirname)(require.resolve((0, _path.join)(cwd, 'node_modules/antd/package')));
    }

    if (dependencies['antd-mobile']) {
      libAlias['antd-mobile'] = (0, _path.dirname)(require.resolve((0, _path.join)(cwd, 'node_modules/antd-mobile/package')));
    }
  }

  const browserslist = webpackRCConfig.browserslist || _browsers.default;
  return (0, _getConfig.default)(_extends({
    cwd
  }, webpackRCConfig, {
    // 不允许配置
    entry,
    outputPath: (0, _path.join)(paths.absOutputPath, staticDirectory),
    hash: !isDev && hash,
    // 扩展
    babel: webpackRCConfig.babel || {
      presets: [[babel, {
        browsers: browserslist
      }]]
    },
    browserslist,
    theme: _extends({}, webpackRCConfig.theme, config.hd ? {
      '@hd': '2px'
    } : {}),
    extraResolveModules: [...(webpackRCConfig.extraResolveModules || []), ...(extraResolveModules || [])],
    define: _extends({
      // 禁用 antd-mobile 升级提醒
      'process.env.DISABLE_ANTD_MOBILE_UPGRADE': true
    }, webpackRCConfig.define || {}),
    alias: _extends({}, preactOrReactAlias, libAlias, config.alias || {}),
    extraPostCSSPlugins: [...(webpackRCConfig.extraPostCSSPlugins || []), ...(config.hd ? [(0, _postcssPluginPx2rem.default)({
      rootValue: 100,
      minPixelValue: 2
    })] : [])]
  }, isDev ? {
    // 生成环境的 publicPath 是服务端把 assets 发布到 cdn 后配到 HTML 里的
    // 开发环境的 publicPath 写死 /static/
    publicPath: webpackRCConfig.publicPath || `/${staticDirectory}/`
  } : {
    commons: webpackRCConfig.commons || [{
      async: '__common',
      children: true,

      minChunks(module, count) {
        if (pageCount <= 2) {
          return count >= pageCount;
        }

        return count >= pageCount * 0.5;
      }

    }]
  }));
}

module.exports = exports["default"];